@@ -20,6 +20,7 @@ export parallel_jobs=1
 # Temporary files, directories, and modules created during tests
 TEST_MODULES=(
     "dkms_test"
+    "dkms_noautoinstall_test"
     "dkms_failing_test"
     "dkms_dependencies_test"
     "dkms_multiver_test"
@@ -31,8 +32,9 @@ TEST_MODULES=(
     "dkms_build_exclusive_dependencies_test"
 )
 TEST_TMPDIRS=(
-    "/usr/src/dkms_test-1.0/"
-    "/usr/src/dkms_failing_test-1.0/"
+    "/usr/src/dkms_test-1.0"
+    "/usr/src/dkms_noautoinstall_test-1.0"
+    "/usr/src/dkms_failing_test-1.0"
     "/usr/src/dkms_dependencies_test-1.0"
     "/usr/src/dkms_multiver_test-1.0"
     "/usr/src/dkms_multiver_test-2.0"
@@ -146,18 +148,12 @@ run_status_with_expected_output() {
     local module=$1
 
     cat > test_cmd_expected_output.log
-    if dkms_status_grep_dkms_module "${module}" > test_cmd_output.log 2>&1 ; then
-        if ! diff -U3 test_cmd_expected_output.log test_cmd_output.log ; then
-            echo >&2 "Error: unexpected output from: dkms_status_grep_dkms_module for ${module}"
-            return 1
-        fi
-        rm test_cmd_expected_output.log test_cmd_output.log
-    else
-        echo "Error: dkms status for ${module} returned status $?"
-        cat test_cmd_output.log
-        rm test_cmd_expected_output.log test_cmd_output.log
+    dkms_status_grep_dkms_module "${module}" > test_cmd_output.log 2>&1
+    if ! diff -U3 test_cmd_expected_output.log test_cmd_output.log ; then
+        echo >&2 "Error: unexpected output from: dkms_status_grep_dkms_module for ${module}"
         return 1
     fi
+    rm test_cmd_expected_output.log test_cmd_output.log
 }
 
 genericize_expected_output() {
@@ -231,23 +227,43 @@ kmod_broken_hashalgo() {
     (( kmod_ver < 26 ))
 }
 
+mod_compression_ext=
+kernel_config="/lib/modules/${KERNEL_VER}/build/.config"
+if [ -f "${kernel_config}" ]; then
+    if grep -q "^CONFIG_MODULE_COMPRESS_NONE=y" "${kernel_config}" ; then
+        mod_compression_ext=
+    elif grep -q "^CONFIG_MODULE_COMPRESS_GZIP=y" "${kernel_config}" ; then
+        mod_compression_ext=.gz
+    elif grep -q "^CONFIG_MODULE_COMPRESS_XZ=y" "${kernel_config}" ; then
+        mod_compression_ext=.xz
+    elif grep -q "^CONFIG_MODULE_COMPRESS_ZSTD=y" "${kernel_config}" ; then
+        mod_compression_ext=.zst
+    fi
+fi
+
 # Compute the expected destination module location
 os_id="$(sed -n 's/^ID\s*=\s*\(.*\)$/\1/p' /etc/os-release | tr -d '"')"
-mod_compression_ext=
 case "${os_id}" in
     centos | fedora | rhel | ovm | almalinux)
         expected_dest_loc=extra
         mod_compression_ext=.xz
         ;;
-    sles | suse | opensuse)
+    sles | suse | opensuse*)
         expected_dest_loc=updates
+        mod_compression_ext=.zst
         ;;
-    arch | debian | ubuntu)
+    arch)
+        expected_dest_loc=updates/dkms
+        ;;
+    debian* | ubuntu | linuxmint)
         expected_dest_loc=updates/dkms
         ;;
     alpine)
         expected_dest_loc=kernel/extra
-        mod_compression_ext=.gz
+        ;;
+    gentoo)
+        expected_dest_loc=kernel/extra
+        mod_compression_ext=
         ;;
     *)
         echo >&2 "Error: unknown Linux distribution ID ${os_id}"
@@ -255,6 +271,11 @@ case "${os_id}" in
         ;;
 esac
 
+echo "Checking module compression ..."
+echo "config: $(grep "^CONFIG_MODULE_COMPRESS" "${kernel_config}" || true)"
+echo "files: $(find  "/lib/modules/${KERNEL_VER}" -name \*.ko\* 2>/dev/null | head -n1)"
+echo "Expected extension: ${mod_compression_ext:-(none)}"
+
 
 echo 'Preparing a clean test environment'
 clean_dkms_env
@@ -306,7 +327,7 @@ run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_test -v 1.0 << EO
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
 run_status_with_expected_output 'dkms_test' << EOF
@@ -329,7 +350,7 @@ Binary /no/such/file not found, modules won't be signed
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 Cleaning build area...
 EOF
 
@@ -340,7 +361,7 @@ Key file /no/such/path.key not found and can't be generated, modules won't be si
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 Cleaning build area...
 EOF
 
@@ -351,7 +372,7 @@ Certificate file /no/such/path.crt not found and can't be generated, modules won
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 Cleaning build area...
 EOF
     rm /tmp/dkms_test_private_key
@@ -363,7 +384,7 @@ EOF
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
     rm -r "/tmp/dkms_test_dir_${KERNEL_VER}/"
@@ -389,7 +410,7 @@ EOF
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
         run_with_expected_output sh -c "modinfo -F sig_hashalgo '${BUILT_MODULE_PATH}'" << EOF
@@ -408,7 +429,7 @@ run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_test -v 1.0 --for
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
 run_status_with_expected_output 'dkms_test' << EOF
@@ -578,7 +599,7 @@ Creating symlink /var/lib/dkms/dkms_test/1.0/source -> /usr/src/dkms_test-1.0
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 
 dkms_test.ko${mod_compression_ext}:
@@ -654,7 +675,7 @@ Creating symlink /var/lib/dkms/dkms_test/1.0/source -> /usr/src/dkms_test-1.0
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
 run_status_with_expected_output 'dkms_test' << EOF
@@ -673,6 +694,9 @@ Running module version sanity check.
 depmod...
 dkms autoinstall on ${KERNEL_VER}/${KERNEL_ARCH} succeeded for dkms_test
 EOF
+run_status_with_expected_output 'dkms_test' << EOF
+dkms_test/1.0, ${KERNEL_VER}, ${KERNEL_ARCH}: installed
+EOF
 
 echo "Running dkms autoinstall for a kernel without headers installed (expected error)"
 run_with_expected_error 11 dkms autoinstall -k "${KERNEL_VER}-noheaders" << EOF
@@ -711,6 +735,104 @@ rm -r /usr/src/dkms_test-1.0
 echo 'Checking that the environment is clean again'
 check_no_dkms_test
 
+############################################################################
+### Testing dkms on a regular module with AUTOINSTALL=""                 ###
+############################################################################
+
+echo 'Adding the noautoinstall test module by directory'
+run_with_expected_output dkms add test/dkms_noautoinstall_test-1.0 << EOF
+Creating symlink /var/lib/dkms/dkms_noautoinstall_test/1.0/source -> /usr/src/dkms_noautoinstall_test-1.0
+EOF
+run_status_with_expected_output 'dkms_noautoinstall_test' << EOF
+dkms_noautoinstall_test/1.0: added
+EOF
+if ! [[ -d /usr/src/dkms_noautoinstall_test-1.0 ]] ; then
+    echo >&2 'Error: directory /usr/src/dkms_noautoinstall_test-1.0 was not created'
+    exit 1
+fi
+
+echo "Running dkms autoinstall"
+run_with_expected_output dkms autoinstall -k "${KERNEL_VER}" << EOF
+EOF
+run_status_with_expected_output 'dkms_noautoinstall_test' << EOF
+dkms_noautoinstall_test/1.0: added
+EOF
+
+echo 'Building the noautoinstall test module'
+set_signing_message "dkms_noautoinstall_test" "1.0"
+run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_noautoinstall_test -v 1.0 << EOF
+
+Building module:
+Cleaning build area...
+Building module(s)...
+${SIGNING_MESSAGE}Cleaning build area...
+EOF
+run_status_with_expected_output 'dkms_noautoinstall_test' << EOF
+dkms_noautoinstall_test/1.0, ${KERNEL_VER}, ${KERNEL_ARCH}: built
+EOF
+
+echo 'Installing the noautoinstall test module'
+run_with_expected_output dkms install -k "${KERNEL_VER}" -m dkms_noautoinstall_test -v 1.0 << EOF
+
+dkms_noautoinstall_test.ko${mod_compression_ext}:
+Running module version sanity check.
+ - Original module
+   - No original module exists within this kernel
+ - Installation
+   - Installing to /lib/modules/${KERNEL_VER}/${expected_dest_loc}/
+depmod...
+EOF
+run_status_with_expected_output 'dkms_noautoinstall_test' << EOF
+dkms_noautoinstall_test/1.0, ${KERNEL_VER}, ${KERNEL_ARCH}: installed
+EOF
+
+echo 'Uninstalling the noautoinstall test module'
+run_with_expected_output dkms uninstall -k "${KERNEL_VER}" -m dkms_noautoinstall_test -v 1.0 << EOF
+Module dkms_noautoinstall_test-1.0 for kernel ${KERNEL_VER} (${KERNEL_ARCH}).
+Before uninstall, this module version was ACTIVE on this kernel.
+
+dkms_noautoinstall_test.ko${mod_compression_ext}:
+ - Uninstallation
+   - Deleting from: /lib/modules/${KERNEL_VER}/${expected_dest_loc}/
+ - Original module
+   - No original module was found for this module on this kernel.
+   - Use the dkms install command to reinstall any previous module version.
+EOF
+run_status_with_expected_output 'dkms_noautoinstall_test' << EOF
+dkms_noautoinstall_test/1.0, ${KERNEL_VER}, ${KERNEL_ARCH}: built
+EOF
+if [[ -e "/lib/modules/${KERNEL_VER}/${expected_dest_loc}/dkms_noautoinstall_test.ko${mod_compression_ext}" ]] ; then
+    echo >&2 "Error: module not removed in /lib/modules/${KERNEL_VER}/${expected_dest_loc}/dkms_noautoinstall_test.ko${mod_compression_ext}"
+    exit 1
+fi
+
+echo 'Unbuilding the noautoinstall test module'
+run_with_expected_output dkms unbuild -k "${KERNEL_VER}" -m dkms_noautoinstall_test -v 1.0 << EOF
+Module dkms_noautoinstall_test 1.0 is not installed for kernel ${KERNEL_VER} (${KERNEL_ARCH}). Skipping...
+EOF
+run_status_with_expected_output 'dkms_noautoinstall_test' << EOF
+dkms_noautoinstall_test/1.0: added
+EOF
+
+echo 'Removing the noautoinstall test module'
+run_with_expected_output dkms remove -k "${KERNEL_VER}" -m dkms_noautoinstall_test -v 1.0 << EOF
+Module dkms_noautoinstall_test 1.0 is not installed for kernel ${KERNEL_VER} (${KERNEL_ARCH}). Skipping...
+Module dkms_noautoinstall_test 1.0 is not built for kernel ${KERNEL_VER} (${KERNEL_ARCH}). Skipping...
+Deleting module dkms_noautoinstall_test-1.0 completely from the DKMS tree.
+EOF
+run_status_with_expected_output 'dkms_noautoinstall_test' << EOF
+EOF
+if ! [[ -d /usr/src/dkms_noautoinstall_test-1.0 ]] ; then
+    echo >&2 'Error: directory /usr/src/dkms_noautoinstall_test-1.0 was removed'
+    exit 1
+fi
+
+echo 'Removing /usr/src/dkms_noautoinstall_test-1.0'
+rm -r /usr/src/dkms_noautoinstall_test-1.0
+
+echo 'Checking that the environment is clean again'
+check_no_dkms_test
+
 ############################################################################
 ### Testing malformed/borderline dkms.conf                               ###
 ############################################################################
@@ -754,10 +876,15 @@ dkms.conf: Warning! Zero modules specified.
 dkms.conf: Warning! Zero modules specified.
 Creating symlink /var/lib/dkms/dkms_conf_test/1.0/source -> /usr/src/dkms_conf_test-1.0
 EOF
+run_status_with_expected_output 'dkms_conf_test' << EOF
+dkms_conf_test/1.0: added
+EOF
 
 run_with_expected_output dkms remove --all -m dkms_conf_test -v 1.0 << EOF
 Deleting module dkms_conf_test-1.0 completely from the DKMS tree.
 EOF
+run_status_with_expected_output 'dkms_conf_test' << EOF
+EOF
 
 echo 'Testing add/build/install of a test module building zero kernel modules'
 run_with_expected_output dkms install -k "${KERNEL_VER}" -m dkms_conf_test -v 1.0 << EOF
@@ -766,7 +893,7 @@ Creating symlink /var/lib/dkms/dkms_conf_test/1.0/source -> /usr/src/dkms_conf_t
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_conf_test/1.0/build...
+Building module(s)...
 Cleaning build area...
 depmod...
 EOF
@@ -843,7 +970,7 @@ run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_multiver_test -v
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_multiver_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
 run_status_with_expected_output 'dkms_multiver_test' << EOF
@@ -855,7 +982,7 @@ run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_multiver_test -v
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_multiver_test/2.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
 run_status_with_expected_output 'dkms_multiver_test' << EOF
@@ -970,6 +1097,9 @@ EOF
 echo 'Removing /usr/src/dkms_multiver_test-1.0 /usr/src/dkms_multiver_test-2.0'
 rm -r /usr/src/dkms_multiver_test-1.0 /usr/src/dkms_multiver_test-2.0
 
+echo 'Checking that the environment is clean again'
+check_no_dkms_test
+
 ############################################################################
 ### Testing dkms operations ...
 ############################################################################
@@ -1002,7 +1132,7 @@ run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_nover_test -v 1.0
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_nover_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
 run_status_with_expected_output 'dkms_nover_test' << EOF
@@ -1013,7 +1143,7 @@ run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_emptyver_test -v
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_emptyver_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
 run_status_with_expected_output 'dkms_emptyver_test' << EOF
@@ -1138,7 +1268,7 @@ run_with_expected_output dkms install -k "${KERNEL_VER}" -m dkms_nover_update_te
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_nover_update_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 
 dkms_nover_update_test.ko${mod_compression_ext}:
@@ -1172,7 +1302,7 @@ run_with_expected_output dkms install -k "${KERNEL_VER}" -m dkms_nover_update_te
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_nover_update_test/2.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 
 dkms_nover_update_test.ko${mod_compression_ext}:
@@ -1208,7 +1338,7 @@ run_with_expected_output dkms build -k "${KERNEL_VER}" -m dkms_nover_update_test
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_nover_update_test/3.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 EOF
 run_status_with_expected_output 'dkms_nover_update_test' << EOF
@@ -1287,7 +1417,9 @@ run_with_expected_error 11 dkms autoinstall -k "${KERNEL_VER}" << EOF
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} all...(bad exit status: 2)
+Building module(s)...(bad exit status: 2)
+Failed command:
+make -j1 KERNELRELEASE=${KERNEL_VER} all
 Error! Bad return status for module build on kernel: ${KERNEL_VER} (${KERNEL_ARCH})
 Consult /var/lib/dkms/dkms_failing_test/1.0/build/make.log for more information.
 dkms autoinstall on ${KERNEL_VER}/${KERNEL_ARCH} failed for dkms_failing_test(10)
@@ -1304,7 +1436,9 @@ run_with_expected_error 11 dkms autoinstall -k "${KERNEL_VER}" << EOF
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} all...(bad exit status: 2)
+Building module(s)...(bad exit status: 2)
+Failed command:
+make -j1 KERNELRELEASE=${KERNEL_VER} all
 Error! Bad return status for module build on kernel: ${KERNEL_VER} (${KERNEL_ARCH})
 Consult /var/lib/dkms/dkms_failing_test/1.0/build/make.log for more information.
 dkms autoinstall on ${KERNEL_VER}/${KERNEL_ARCH} failed for dkms_failing_test(10)
@@ -1363,7 +1497,9 @@ fi
 # Should this really fail?
 echo '(Not) building the build-exclusive test module'
 run_with_expected_error 77 dkms build -k "${KERNEL_VER}" -m dkms_build_exclusive_test -v 1.0 << EOF
-Error! The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive which does not match this kernel/arch/config.
+Warning: The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf
+for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive
+which does not match this kernel/arch/config.
 This indicates that it should not be built.
 EOF
 run_status_with_expected_output 'dkms_build_exclusive_test' << EOF
@@ -1372,7 +1508,9 @@ EOF
 
 echo "Running dkms autoinstall (1 x skip)"
 run_with_expected_output dkms autoinstall -k "${KERNEL_VER}" << EOF
-Error! The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive which does not match this kernel/arch/config.
+Warning: The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf
+for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive
+which does not match this kernel/arch/config.
 This indicates that it should not be built.
 dkms autoinstall on ${KERNEL_VER}/${KERNEL_ARCH} was skipped for dkms_build_exclusive_test
 EOF
@@ -1390,12 +1528,14 @@ EOF
 
 echo "Running dkms autoinstall (1 x skip, 1 x pass)"
 run_with_expected_output dkms autoinstall -k "${KERNEL_VER}" << EOF
-Error! The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive which does not match this kernel/arch/config.
+Warning: The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf
+for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive
+which does not match this kernel/arch/config.
 This indicates that it should not be built.
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 
 dkms_test.ko${mod_compression_ext}:
@@ -1438,18 +1578,22 @@ EOF
 
 echo "Running dkms autoinstall (1 x skip, 1 x fail, 1 x pass) (expected error)"
 run_with_expected_error 11 dkms autoinstall -k "${KERNEL_VER}" << EOF
-Error! The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive which does not match this kernel/arch/config.
+Warning: The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf
+for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive
+which does not match this kernel/arch/config.
 This indicates that it should not be built.
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} all...(bad exit status: 2)
+Building module(s)...(bad exit status: 2)
+Failed command:
+make -j1 KERNELRELEASE=${KERNEL_VER} all
 Error! Bad return status for module build on kernel: ${KERNEL_VER} (${KERNEL_ARCH})
 Consult /var/lib/dkms/dkms_failing_test/1.0/build/make.log for more information.
 
 Building module:
 Cleaning build area...
-make -j1 KERNELRELEASE=${KERNEL_VER} -C /lib/modules/${KERNEL_VER}/build M=/var/lib/dkms/dkms_test/1.0/build...
+Building module(s)...
 ${SIGNING_MESSAGE}Cleaning build area...
 
 dkms_test.ko${mod_compression_ext}:
@@ -1507,9 +1651,13 @@ fi
 
 echo "Running dkms autoinstall (2 x skip, with dependency)"
 run_with_expected_output dkms autoinstall -k "${KERNEL_VER}" << EOF
-Error! The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive which does not match this kernel/arch/config.
+Warning: The /var/lib/dkms/dkms_build_exclusive_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf
+for module dkms_build_exclusive_test includes a BUILD_EXCLUSIVE directive
+which does not match this kernel/arch/config.
 This indicates that it should not be built.
-Error! The /var/lib/dkms/dkms_build_exclusive_dependencies_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf for module dkms_build_exclusive_dependencies_test includes a BUILD_EXCLUSIVE directive which does not match this kernel/arch/config.
+Warning: The /var/lib/dkms/dkms_build_exclusive_dependencies_test/1.0/${KERNEL_VER}/${KERNEL_ARCH}/dkms.conf
+for module dkms_build_exclusive_dependencies_test includes a BUILD_EXCLUSIVE directive
+which does not match this kernel/arch/config.
 This indicates that it should not be built.
 dkms autoinstall on ${KERNEL_VER}/${KERNEL_ARCH} was skipped for dkms_build_exclusive_test dkms_build_exclusive_dependencies_test
 EOF
@@ -1541,4 +1689,275 @@ rm -r /usr/src/dkms_build_exclusive_test-1.0
 echo 'Checking that the environment is clean again'
 check_no_dkms_test
 
+############################################################################
+### Testing os-release detection                                         ###
+############################################################################
+echo "Backing up /etc/os-release and /usr/lib/os-release"
+osrelease_cleanup() {
+    rm -f _os-release
+    mv _etc-os-release /etc/os-release &>/dev/null || :
+    mv _usrlib-os-release /usr/lib/os-release &>/dev/null || :
+}
+
+for f in /etc/os-release /usr/lib/os-release; do
+    if [ -e "$f" ]; then
+        cp --preserve=all -f "$f" _os-release
+        break
+    fi
+done
+[ -f _os-release ] || { echo >&2 "Error: file os-release not found"; exit 1; }
+trap osrelease_cleanup EXIT
+
+mv_osrelease() {
+    if [ -f "$1" ]; then
+       mv "$1" "$2" || { echo >&2 "Error: could not move os-release $1"; exit 1; }
+    fi
+}
+mv_osrelease "/etc/os-release" "_etc-os-release"
+mv_osrelease "/usr/lib/os-release" "_usrlib-os-release"
+
+echo "Adding the dkms_test-1.0 module with no os-release files (expected error)"
+run_with_expected_error 4 dkms add test/dkms_test-1.0 << EOF
+Error! System is missing os-release file.
+EOF
+
+echo "Creating /etc/os-release"
+cp -f _os-release /etc/os-release
+echo "Adding the dkms_test-1.0 module with file /etc/os-release"
+run_with_expected_output dkms add test/dkms_test-1.0 << EOF
+Creating symlink /var/lib/dkms/dkms_test/1.0/source -> /usr/src/dkms_test-1.0
+EOF
+run_status_with_expected_output 'dkms_test' << EOF
+dkms_test/1.0: added
+EOF
+
+echo 'Removing dkms_test module'
+run_with_expected_output dkms remove -k "${KERNEL_VER}" -m dkms_test -v 1.0 << EOF
+Module dkms_test 1.0 is not installed for kernel ${KERNEL_VER} (${KERNEL_ARCH}). Skipping...
+Module dkms_test 1.0 is not built for kernel ${KERNEL_VER} (${KERNEL_ARCH}). Skipping...
+Deleting module dkms_test-1.0 completely from the DKMS tree.
+EOF
+echo "Removing /usr/src/dkms_test-1.0"
+rm -r /usr/src/dkms_test-1.0
+echo "Deleting /etc/os-release"
+rm -f /etc/os-release
+
+echo "Creating /usr/lib/os-release"
+cp -f _os-release /etc/os-release
+echo "Adding the dkms_test-1.0 module with file /usr/lib/os-release"
+run_with_expected_output dkms add test/dkms_test-1.0 << EOF
+Creating symlink /var/lib/dkms/dkms_test/1.0/source -> /usr/src/dkms_test-1.0
+EOF
+run_status_with_expected_output 'dkms_test' << EOF
+dkms_test/1.0: added
+EOF
+
+echo 'Removing dkms_test module'
+run_with_expected_output dkms remove -k "${KERNEL_VER}" -m dkms_test -v 1.0 << EOF
+Module dkms_test 1.0 is not installed for kernel ${KERNEL_VER} (${KERNEL_ARCH}). Skipping...
+Module dkms_test 1.0 is not built for kernel ${KERNEL_VER} (${KERNEL_ARCH}). Skipping...
+Deleting module dkms_test-1.0 completely from the DKMS tree.
+EOF
+echo "Removing /usr/src/dkms_test-1.0"
+rm -r /usr/src/dkms_test-1.0
+echo "Deleting /usr/lib/os-release"
+rm -f /usr/lib/os-release
+
+echo "Restoring /etc/os-release and /usr/bin/os-release"
+osrelease_cleanup
+trap - EXIT
+
+echo 'Checking that the environment is clean again'
+check_no_dkms_test
+
+############################################################################
+### Testing 'broken' status                                              ###
+############################################################################
+
+echo
+echo 'Running BROKEN tests'
+echo
+
+echo 'Adding the test module by directory'
+run_with_expected_output dkms add test/dkms_test-1.0 << EOF
+Creating symlink /var/lib/dkms/dkms_test/1.0/source -> /usr/src/dkms_test-1.0
+EOF
+
+echo ' Removing symlink /var/lib/dkms/dkms_test/1.0/source'
+rm /var/lib/dkms/dkms_test/1.0/source
+
+echo 'Checking broken status'
+run_with_expected_output dkms status dkms_test/1.0 << EOF
+dkms_test/1.0: broken
+Error! dkms_test/1.0: Missing the module source directory or the symbolic link pointing to it.
+Manual intervention is required!
+EOF
+
+echo 'Re-adding the test module'
+run_with_expected_output dkms add dkms_test/1.0 << EOF
+Creating symlink /var/lib/dkms/dkms_test/1.0/source -> /usr/src/dkms_test-1.0
+EOF
+
+echo ' Removing symlink /var/lib/dkms/dkms_test/1.0/source'
+rm /var/lib/dkms/dkms_test/1.0/source
+
+echo 'Building broken test module (expected erorr)'
+run_with_expected_error 4 dkms build dkms_test/1.0 << EOF
+Error! dkms_test/1.0 is broken!
+Missing the source directory or the symbolic link pointing to it.
+Manual intervention is required!
+EOF
+
+echo 'Installing broken test module (expected erorr)'
+run_with_expected_error 4 dkms install dkms_test/1.0 << EOF
+Error! dkms_test/1.0 is broken!
+Missing the source directory or the symbolic link pointing to it.
+Manual intervention is required!
+EOF
+
+echo 'Unbuild broken test module (expected erorr)'
+run_with_expected_error 4 dkms unbuild dkms_test/1.0 << EOF
+Error! dkms_test/1.0 is broken!
+Missing the source directory or the symbolic link pointing to it.
+Manual intervention is required!
+EOF
+
+echo 'Uninstall broken test module (expected erorr)'
+run_with_expected_error 4 dkms uninstall dkms_test/1.0 << EOF
+Error! dkms_test/1.0 is broken!
+Missing the source directory or the symbolic link pointing to it.
+Manual intervention is required!
+EOF
+
+echo 'Adding the multiver test module 1.0 by directory'
+run_with_expected_output dkms add test/dkms_multiver_test/1.0 << EOF
+Creating symlink /var/lib/dkms/dkms_multiver_test/1.0/source -> /usr/src/dkms_multiver_test-1.0
+EOF
+
+echo 'Checking broken status'
+run_with_expected_output dkms status << EOF
+dkms_multiver_test/1.0: added
+dkms_test/1.0: broken
+Error! dkms_test/1.0: Missing the module source directory or the symbolic link pointing to it.
+Manual intervention is required!
+EOF
+
+echo 'Remove broken test module (expected erorr)'
+run_with_expected_error 4 dkms remove dkms_test/1.0 << EOF
+Error! dkms_test/1.0 is broken!
+Missing the source directory or the symbolic link pointing to it.
+Manual intervention is required!
+EOF
+
+echo 'Re-adding the test module'
+run_with_expected_output dkms add dkms_test/1.0 << EOF
+Creating symlink /var/lib/dkms/dkms_test/1.0/source -> /usr/src/dkms_test-1.0
+EOF
+
+echo ' Removing source tree /usr/src/dkms_test-1.0/'
+rm -rf /usr/src/dkms_test-1.0/
+
+echo 'Checking broken status'
+run_with_expected_output dkms status << EOF
+dkms_multiver_test/1.0: added
+dkms_test/1.0: broken
+Error! dkms_test/1.0: Missing the module source directory or the symbolic link pointing to it.
+Manual intervention is required!
+EOF
+
+echo 'Removing dkms_multiver_test'
+dkms remove dkms_multiver_test/1.0 -k "${KERNEL_VER}" > /dev/null
+
+echo 'Removing dkms_test'
+rm -rf /var/lib/dkms/dkms_test/
+
+echo 'Adding and building the test module by directory'
+set_signing_message "dkms_test" "1.0"
+run_with_expected_output dkms build test/dkms_test-1.0 -k "${KERNEL_VER}" << EOF
+Creating symlink /var/lib/dkms/dkms_test/1.0/source -> /usr/src/dkms_test-1.0
+
+Building module:
+Cleaning build area...
+Building module(s)...
+${SIGNING_MESSAGE}Cleaning build area...
+EOF
+
+echo 'Adding and building the multiver test module 1.0 by directory'
+set_signing_message "dkms_multiver_test" "1.0"
+run_with_expected_output dkms build test/dkms_multiver_test/1.0 -k "${KERNEL_VER}" << EOF
+Creating symlink /var/lib/dkms/dkms_multiver_test/1.0/source -> /usr/src/dkms_multiver_test-1.0
+
+Building module:
+Cleaning build area...
+Building module(s)...
+${SIGNING_MESSAGE}Cleaning build area...
+EOF
+
+echo ' Removing symlink /var/lib/dkms/dkms_multiver_test/1.0/source'
+rm /var/lib/dkms/dkms_multiver_test/1.0/source
+
+echo 'Adding and building the multiver test module 2.0 by directory'
+set_signing_message "dkms_multiver_test" "2.0"
+run_with_expected_output dkms build test/dkms_multiver_test/2.0 -k "${KERNEL_VER}" << EOF
+Creating symlink /var/lib/dkms/dkms_multiver_test/2.0/source -> /usr/src/dkms_multiver_test-2.0
+
+Building module:
+Cleaning build area...
+Building module(s)...
+${SIGNING_MESSAGE}Cleaning build area...
+EOF
+
+echo 'Running dkms autoinstall'
+run_with_expected_output dkms autoinstall -k "${KERNEL_VER}" << EOF
+Error! dkms_multiver_test/1.0 is broken! Missing the source directory or the symbolic link pointing to it.
+Manual intervention is required!
+
+dkms_test.ko${mod_compression_ext}:
+Running module version sanity check.
+ - Original module
+   - No original module exists within this kernel
+ - Installation
+   - Installing to /lib/modules/${KERNEL_VER}/${expected_dest_loc}/
+depmod...
+dkms autoinstall on ${KERNEL_VER}/${KERNEL_ARCH} succeeded for dkms_test
+EOF
+run_with_expected_output dkms status << EOF
+dkms_multiver_test/1.0: broken
+Error! dkms_multiver_test/1.0: Missing the module source directory or the symbolic link pointing to it.
+Manual intervention is required!
+dkms_multiver_test/2.0, ${KERNEL_VER}, ${KERNEL_ARCH}: built
+dkms_test/1.0, ${KERNEL_VER}, ${KERNEL_ARCH}: installed
+EOF
+
+echo 'Removing all modules'
+echo ' Removing the test module'
+run_with_expected_output dkms remove dkms_test/1.0 -k "${KERNEL_VER}" << EOF
+Module dkms_test-1.0 for kernel ${KERNEL_VER} (${KERNEL_ARCH}).
+Before uninstall, this module version was ACTIVE on this kernel.
+
+dkms_test.ko${mod_compression_ext}:
+ - Uninstallation
+   - Deleting from: /lib/modules/${KERNEL_VER}/${expected_dest_loc}/
+ - Original module
+   - No original module was found for this module on this kernel.
+   - Use the dkms install command to reinstall any previous module version.
+Deleting module dkms_test-1.0 completely from the DKMS tree.
+EOF
+
+echo ' Removing the multi_ver_test 2.0 module'
+run_with_expected_output dkms remove -m dkms_multiver_test -v 2.0 -k "${KERNEL_VER}" << EOF
+Module dkms_multiver_test 2.0 is not installed for kernel ${KERNEL_VER} (${KERNEL_ARCH}). Skipping...
+Deleting module dkms_multiver_test-2.0 completely from the DKMS tree.
+EOF
+
+echo ' Removing directories: /var/lib/dkms/dkms_test/ /var/lib/dkms/dkms_multiver_test /usr/src/dkms_test-1.0 /usr/src/dkms_multiver_test-?.0'
+rm -rf /var/lib/dkms/dkms_test/ /var/lib/dkms/dkms_multiver_test /usr/src/dkms_test-1.0 /usr/src/dkms_multiver_test-?.0
+
+echo 'Checking that the environment is clean again'
+check_no_dkms_test
+
+echo
+echo 'End of BROKEN tests'
+echo
+
 echo 'All tests successful :)'
